import { connect } from 'react-redux'
import { actions } from 'pubsweet-client'
import { ConnectPage } from 'xpub-connect'
import { withJournal } from 'xpub-journal'
import { getFormValues } from 'redux-form'
import { replace } from 'react-router-redux'
import { withRouter } from 'react-router-dom'
import { head, get, isEmpty, isUndefined } from 'lodash'
import {
  selectFragment,
  selectCollection,
  selectCurrentUser,
} from 'xpub-selectors'
import { get as apiGet } from 'pubsweet-client/src/helpers/api'
import {
  compose,
  withState,
  lifecycle,
  withProps,
  withHandlers,
  setDisplayName,
  fromRenderProps,
} from 'recompose'
import { getSignedUrl } from 'pubsweet-components-faraday/src/redux/files'
import {
  inviteReviewer,
  revokeReviewer,
  reviewerDecision,
} from 'pubsweet-components-faraday/src/redux/reviewers'
import {
  hasManuscriptFailure,
  clearCustomError,
} from 'pubsweet-components-faraday/src/redux/errors'
import { selectEditorialRecommendations } from 'pubsweet-components-faraday/src/redux/recommendations'
import {
  getUserToken,
  currentUserIs,
  canMakeRevision,
  canMakeDecision,
  canEditManuscript,
  canInviteReviewers,
  pendingHEInvitation,
  currentUserIsReviewer,
  canMakeRecommendation,
  parseCollectionDetails,
  pendingReviewerInvitation,
  canOverrideTechnicalChecks,
  getInvitationsWithReviewersForFragment,
} from 'pubsweet-component-faraday-selectors'
import { RemoteOpener, handleError } from 'pubsweet-component-faraday-ui'

import ManuscriptLayout from './ManuscriptLayout'
import { parseEicDecision, parseSearchParams, redirectToError } from './utils'
import {
  canAssignHE,
  selectFetching,
  getHandlingEditors,
  assignHandlingEditor,
  revokeHandlingEditor,
  selectHandlingEditors,
  handlingEditorDecision,
} from '../redux/editors'
import {
  createRecommendation,
  recommendationsFetching,
} from '../redux/recommendations'

export default compose(
  setDisplayName('ManuscriptPage'),
  withJournal,
  withRouter,
  withState('editorInChief', 'setEiC', 'N/A'),
  ConnectPage(({ match }) => [
    actions.getCollection({ id: match.params.project }),
    actions.getFragments({ id: match.params.project }),
    actions.getUsers(),
  ]),
  connect(
    (state, { match }) => ({
      currentUser: selectCurrentUser(state),
      handlingEditors: selectHandlingEditors(state),
      hasManuscriptFailure: hasManuscriptFailure(state),
      fragment: selectFragment(state, match.params.version),
      collection: parseCollectionDetails(
        state,
        selectCollection(state, match.params.project),
      ),
      pendingHEInvitation: pendingHEInvitation(state, match.params.project),
      pendingReviewerInvitation: pendingReviewerInvitation(
        state,
        match.params.version,
      ),
      editorialRecommendations: selectEditorialRecommendations(
        state,
        match.params.version,
      ),
    }),
    {
      replace,
      getSignedUrl,
      clearCustomError,
      assignHandlingEditor,
      createRecommendation,
      revokeHandlingEditor,
      getUsers: actions.getUsers,
      getFragment: actions.getFragment,
      getCollection: actions.getCollection,
      updateVersion: actions.updateFragment,
    },
  ),
  connect(
    (
      state,
      {
        match,
        fragment,
        collection,
        currentUser,
        pendingHEInvitation,
        pendingReviewerInvitation,
      },
    ) => ({
      currentUser: {
        ...currentUser,
        token: getUserToken(state),
        isHE: currentUserIs(state, 'isHE'),
        isEIC: currentUserIs(state, 'adminEiC'),
        isReviewer: currentUserIsReviewer(state),
        isInvitedHE: !isUndefined(pendingHEInvitation),
        isInvitedToReview: !isUndefined(pendingReviewerInvitation),
        permissions: {
          canAssignHE: canAssignHE(state, match.params.project),
          canInviteReviewers: canInviteReviewers(state, collection),
          canMakeRevision: canMakeRevision(state, collection, fragment),
          canMakeDecision: canMakeDecision(state, collection, fragment),
          canEditManuscript: canEditManuscript(state, collection, fragment),
          canOverrideTechChecks: canOverrideTechnicalChecks(state, collection),
          canMakeRecommendation: canMakeRecommendation(
            state,
            collection,
            fragment,
          ),
        },
      },
      isFetching: {
        editorsFetching: selectFetching(state),
        recommendationsFetching: recommendationsFetching(state),
      },
      formValues: {
        eicDecision: getFormValues('eic-decision')(state),
        responseToInvitation: getFormValues('answer-invitation')(state),
      },
      invitationsWithReviewers: getInvitationsWithReviewersForFragment(
        state,
        get(fragment, 'id', ''),
      ),
    }),
  ),
  ConnectPage(({ currentUser }) => {
    if (currentUser.isEIC) {
      return [getHandlingEditors()]
    }
    return []
  }),
  withHandlers({
    fetchUpdatedCollection: ({
      fragment,
      getUsers,
      collection,
      getFragment,
      getCollection,
    }) => () => {
      getCollection({ id: collection.id })
      getFragment(collection, fragment)
      getUsers()
    },
  }),
  withHandlers({
    updateManuscript: ({ updateVersion, collection, fragment }) => data =>
      updateVersion(collection, {
        id: fragment.id,
        ...data,
      }),
    setEditorInChief: ({ setEiC }) => eic => {
      if (eic) {
        const { firstName = '', lastName = '' } = eic
        setEiC(`${firstName} ${lastName}`)
      }
    },
    assignHE: ({
      assignHandlingEditor,
      fetchUpdatedCollection,
      collection: { id: collectionId },
    }) => (email, modalProps) =>
      assignHandlingEditor({
        email,
        collectionId,
      })
        .then(() => {
          fetchUpdatedCollection()
          modalProps.hideModal()
        })
        .catch(handleError(modalProps.setModalError)),
    revokeHE: ({
      getCollection,
      revokeHandlingEditor,
      collection: { id: collectionId },
    }) => (invitationId, modalProps) =>
      revokeHandlingEditor({
        invitationId,
        collectionId,
      })
        .then(() => {
          getCollection({ id: collectionId })
          modalProps.hideModal()
        })
        .catch(handleError(modalProps.setModalError)),
    createRecommendation: ({
      fragment,
      collection,
      fetchUpdatedCollection,
      createRecommendation,
    }) => (values, modalProps) => {
      const recommendation = parseEicDecision(values)
      createRecommendation({
        recommendation,
        fragmentId: fragment.id,
        collectionId: collection.id,
      })
        .then(() => {
          fetchUpdatedCollection()
          modalProps.hideModal()
        })
        .catch(handleError(modalProps.setModalError))
    },
    onHEResponse: ({
      history,
      collection,
      pendingHEInvitation,
      fetchUpdatedCollection,
    }) => (values, { hideModal, setModalError, setFetching }) => {
      const isAccepted = get(values, 'decision', 'decline') === 'accept'
      setFetching(true)
      return handlingEditorDecision({
        isAccepted,
        collectionId: collection.id,
        reason: get(values, 'reason', ''),
        invitationId: pendingHEInvitation.id,
      })
        .then(() => {
          setFetching(false)
          hideModal()
          if (isAccepted) {
            fetchUpdatedCollection()
          } else {
            history.replace('/')
          }
        })
        .catch(() => {
          setFetching(false)
          setModalError('Something went wrong...')
        })
    },
    onReviewerResponse: ({
      history,
      fragment,
      collection,
      fetchUpdatedCollection,
      pendingReviewerInvitation,
    }) => (values, { hideModal, setModalError, setFetching }) => {
      const isAccepted = get(values, 'decision', 'decline') === 'accept'
      setFetching(true)
      reviewerDecision({
        agree: isAccepted,
        fragmentId: fragment.id,
        collectionId: collection.id,
        invitationId: pendingReviewerInvitation.id,
      })
        .then(() => {
          setFetching(false)
          hideModal()
          if (isAccepted) {
            fetchUpdatedCollection()
          } else {
            history.replace('/')
          }
        })
        .catch(() => {
          setFetching(false)
          setModalError('Something went wrong...')
        })
    },
    onInviteReviewer: ({ collection, fragment, fetchUpdatedCollection }) => (
      values,
      { hideModal, setModalError, setFetching },
    ) => {
      setFetching(true)
      inviteReviewer({
        reviewerData: values,
        fragmentId: fragment.id,
        collectionId: collection.id,
      })
        .then(() => {
          setFetching(false)
          hideModal()
          fetchUpdatedCollection()
        })
        .catch(() => {
          setFetching(false)
          setModalError('Something went wrong...')
        })
    },
    onResendReviewerInvite: ({
      fragment,
      collection,
      fetchUpdatedCollection,
    }) => (email, { hideModal, setFetching, setModalError }) => {
      setFetching(true)
      inviteReviewer({
        reviewerData: {
          email,
          role: 'reviewer',
        },
        fragmentId: fragment.id,
        collectionId: collection.id,
      })
        .then(() => {
          setFetching(false)
          hideModal()
          fetchUpdatedCollection()
        })
        .catch(() => {
          setFetching(false)
          setModalError('Something went wrong...')
        })
    },
    onRevokeReviewerInvite: ({
      fragment,
      collection,
      fetchUpdatedCollection,
    }) => (invitationId, { hideModal, setFetching, setModalError }) => {
      setFetching(true)
      revokeReviewer({
        invitationId,
        fragmentId: fragment.id,
        collectionId: collection.id,
      })
        .then(() => {
          setFetching(false)
          hideModal()
          fetchUpdatedCollection()
        })
        .catch(() => {
          setFetching(false)
          setModalError('Something went wrong...')
        })
    },
  }),
  fromRenderProps(RemoteOpener, ({ toggle, expanded }) => ({
    toggleAssignHE: toggle,
    heExpanded: expanded,
  })),
  fromRenderProps(RemoteOpener, ({ toggle, expanded }) => ({
    toggleHEResponse: toggle,
    heResponseExpanded: expanded,
  })),
  fromRenderProps(RemoteOpener, ({ toggle, expanded }) => ({
    toggleReviewerResponse: toggle,
    reviewerResponseExpanded: expanded,
  })),
  lifecycle({
    componentDidMount() {
      const {
        match,
        replace,
        history,
        location,
        setEditorInChief,
        clearCustomError,
        hasManuscriptFailure,
        fetchUpdatedCollection,
        currentUser: { isInvitedHE, isInvitedToReview },
      } = this.props
      if (hasManuscriptFailure) {
        history.push('/not-found')
        clearCustomError()
      }

      const collectionId = match.params.project
      const fragmentId = match.params.version
      const { agree, invitationId } = parseSearchParams(location.search)
      if (agree === 'true') {
        replace(location.pathname)
        reviewerDecision({ invitationId, collectionId, fragmentId })
          .then(fetchUpdatedCollection)
          .catch(redirectToError(replace))
      }

      apiGet(`/users?editorInChief=true`).then(res =>
        setEditorInChief(head(res.users)),
      )

      if (isInvitedHE) {
        this.props.toggleHEResponse()
      }

      if (isInvitedToReview) {
        this.props.toggleReviewerResponse()
      }
    },
  }),
  withProps(({ fragment }) => ({
    hasResponseToReviewers:
      !isEmpty(get(fragment, 'files.responseToReviewers')) ||
      get(fragment, 'commentsToReviewers'),
  })),
)(ManuscriptLayout)
